@@ -3,8 +3,8 @@
   FILE: icalrecur.c
   CREATOR: eric 16 May 2000
   
-  $Id: icalrecur.c,v 1.14 2005/08/11 00:00:25 tml Exp $
-  $Locker:  $
+  $Id: icalrecur.c 604 2007-05-25 14:05:33Z rburton $
+  $Locker$
     
 
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -582,7 +582,7 @@ char* icalrecurrencetype_as_string(struct icalrecurrencetype *recur)
     }
 
     /* If week start is not monday (the default per RFC2445) append WKST */
-    if (recur->week_start != ICAL_MONDAY_WEEKDAY) {
+    if (recur->week_start != ICAL_MONDAY_WEEKDAY && recur->week_start != ICAL_NO_WEEKDAY) {
 	sprintf(temp,"%s", icalrecur_weekday_to_string(recur->week_start));
 	icalmemory_append_string(&str,&str_p,&buf_sz,";WKST=");
 	icalmemory_append_string(&str,&str_p,&buf_sz, temp);
@@ -666,7 +666,7 @@ struct expand_split_map_struct
 	short map[8];
 }; 
 
-static struct expand_split_map_struct expand_map[] =
+static const struct expand_split_map_struct expand_map[] =
 {
     {ICAL_SECONDLY_RECURRENCE,{1,1,1,1,1,1,1,1}},
     {ICAL_MINUTELY_RECURRENCE,{2,1,1,1,1,1,1,1}},
