@@ -38,12 +38,17 @@
 #include <libedataserverui/e-book-auth-util.h>
 #include <libedataserver/e-sexp.h>
 
+#include <libedataserverui/e-data-server-ui-marshal.h>
 #include "e-name-selector-entry.h"
 
+enum {
+	UPDATED,
+	LAST_SIGNAL
+};
+static guint signals[LAST_SIGNAL] = { 0 };
+static guint COMPLETION_CUE_MIN_LEN = 0;
 #define ENS_DEBUG(x)
 
-#define COMPLETION_CUE_MIN_LEN 3
-
 G_DEFINE_TYPE (ENameSelectorEntry, e_name_selector_entry, GTK_TYPE_ENTRY);
 
 typedef struct _ENameSelectorEntryPrivate	ENameSelectorEntryPrivate;
@@ -120,6 +125,12 @@ e_name_selector_entry_finalize (GObject *object)
 		G_OBJECT_CLASS (e_name_selector_entry_parent_class)->finalize (object);
 }
 
+static void
+e_name_selector_entry_updated (ENameSelectorEntry *entry, char *email)
+{
+	g_return_if_fail (E_IS_NAME_SELECTOR_ENTRY (entry));
+}
+
 static void
 e_name_selector_entry_class_init (ENameSelectorEntryClass *name_selector_entry_class)
 {
@@ -130,12 +141,22 @@ e_name_selector_entry_class_init (ENameSelectorEntryClass *name_selector_entry_c
 	object_class->set_property = e_name_selector_entry_set_property;
 	object_class->dispose      = e_name_selector_entry_dispose;
 	object_class->finalize     = e_name_selector_entry_finalize;
+	name_selector_entry_class->updated	   = e_name_selector_entry_updated;
 
 	widget_class->realize      = e_name_selector_entry_realize;
 
 	/* Install properties */
 
 	/* Install signals */
+	
+	signals[UPDATED] = g_signal_new ("updated",
+					 E_TYPE_NAME_SELECTOR_ENTRY,
+					 G_SIGNAL_RUN_FIRST,
+					 G_STRUCT_OFFSET (ENameSelectorEntryClass, updated),
+					 NULL,
+					 NULL,
+					 e_data_server_ui_marshal_VOID__POINTER,
+					 G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	g_type_class_add_private (object_class, sizeof(ENameSelectorEntryPrivate));
 }
@@ -220,6 +241,9 @@ get_range_at_position (const gchar *string, gint pos, gint *start_pos, gint *end
 	gint         local_end_pos   = 0;
 	gint         i;
 
+	if (!string || !*string)
+		return FALSE;
+
 	for (p = string, i = 0; *p; p = g_utf8_next_char (p), i++) {
 		gunichar c = g_utf8_get_char (p);
 
@@ -514,12 +538,6 @@ utf8_casefold_collate_len (const gchar *str1, const gchar *str2, gint len)
 	return rv;
 }
 
-static gint
-utf8_casefold_collate (const gchar *str1, const gchar *str2)
-{
-	return utf8_casefold_collate_len (str1, str2, -1);
-}
-
 static gchar *
 build_textrep_for_contact (EContact *contact, EContactField cue_field)
 {
@@ -552,7 +570,7 @@ build_textrep_for_contact (EContact *contact, EContactField cue_field)
 	g_assert (strlen (email) > 0);
 
 	if (name)
-		textrep = g_strdup_printf ("%s", name);
+		textrep = g_strdup_printf ("%s <%s>", name, email);
 	else
 		textrep = g_strdup_printf ("%s", email);
 
@@ -899,8 +917,9 @@ modify_destination_at_position (ENameSelectorEntry *name_selector_entry, gint po
 	gboolean      rebuild_attributes = FALSE;
 
 	destination = find_destination_at_position (name_selector_entry, pos);
-	g_assert (destination);
-
+	if (!destination)
+		return;
+	
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	raw_address = get_address_at_position (text, pos);
 	g_assert (raw_address);
@@ -920,22 +939,6 @@ modify_destination_at_position (ENameSelectorEntry *name_selector_entry, gint po
 		generate_attribute_list (name_selector_entry);
 }
 
-static void
-remove_destination_at_position (ENameSelectorEntry *name_selector_entry, gint pos)
-{
-	EDestination *destination;
-
-	destination = find_destination_at_position (name_selector_entry, pos);
-	if (destination) {
-		g_signal_handlers_block_by_func (name_selector_entry->destination_store,
-					 destination_row_deleted, name_selector_entry);
-		e_destination_store_remove_destination (name_selector_entry->destination_store,
-						destination);
-		g_signal_handlers_unblock_by_func (name_selector_entry->destination_store,
-					   destination_row_deleted, name_selector_entry);
-	}
-}
-
 static void
 sync_destination_at_position (ENameSelectorEntry *name_selector_entry, gint range_pos, gint *cursor_pos)
 {
@@ -1139,10 +1142,10 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	const gchar *text;
 	gint         index_start, index_end;
 	gint	     selection_start, selection_end;	
-	gunichar     str_context [2], str_b_context [2];;
+	gunichar     str_context [2], str_b_context [2];
 	gint         len;
 	gint         i;
-	gboolean     already_selected = FALSE;
+	gboolean     already_selected = FALSE, del_space = FALSE, del_comma = FALSE;
 
 	if (start_pos == end_pos) 
 		return;
@@ -1153,11 +1156,12 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (name_selector_entry), 
 					       &selection_start, 
 					       &selection_end)) 
-		if (g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end - 1)) == ',') 
+		if ((g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end)) == 0) ||
+		    (g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end)) == ','))
 			already_selected = TRUE;
 	
 	get_utf8_string_context (text, start_pos, str_context, 2);
-	get_utf8_string_context (text, end_pos-1, str_b_context, 2);
+	get_utf8_string_context (text, end_pos, str_b_context, 2);
 
 	g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
 
@@ -1175,47 +1179,120 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	
 	g_signal_stop_emission_by_name (name_selector_entry, "delete_text");
 	
-	/* If the user is trying to delete a ','-character, we assume the user 
-	 * wants to remove the entire destination.
-	 */
-
-	if (((str_b_context [0] == ',' && str_b_context [1] == ' ') || str_b_context [1] == ',') && !already_selected) {
-
-		EDestination *dest = find_destination_at_position (name_selector_entry, end_pos-1);
-	  	const char *email = e_destination_get_email (dest);
-	  	if (email && (strcmp (email, "")!=0)) {
-	  
-			/* Therefore, in case it's a real destination, we select it. 
-		 	 * Deleting this selection afterwards will leave the destination
-		 	 * empty. */
-	 
-			gint t = (str_b_context [1]==',')?end_pos:end_pos-1, b=t;
-			do {
-				t--;
-			} while (t >= 1 && text[t-1] != ',');
-	
-			gtk_editable_select_region (GTK_EDITABLE(name_selector_entry), t, b);
-
-			/* Since this is a special-case, we don't want the rest of this method
-		 	 * to happen. However, we do need to reenable the signal which we
-		 	 * disabled above! */
-		
-			goto end_of_user_delete_text;
-	  	}
-	}
-	
 	/* If the deletion touches more than one destination, the first one is changed
 	 * and the rest are removed. If the last destination wasn't completely deleted,
 	 * it becomes part of the first one, since the separator between them was
 	 * removed.
 	 *
 	 * Here, we let the model know about removals. */
-	for (i = index_end; i > index_start; i--)
+	for (i = index_end; i > index_start; i--) {
+		EDestination *destination = find_destination_by_index (name_selector_entry, i);
+		int range_start, range_end;
+		char *ttext;
+		const char *email=NULL;
+		gboolean sel=FALSE;
+
+		if (destination)
+			email = e_destination_get_address (destination);
+
+		if (!email || !*email)
+			continue;
+		
+		if (!get_range_by_index (text, i, &range_start, &range_end)) {
+			g_warning ("ENameSelectorEntry is out of sync with model!");
+			return;
+		}
+
+		if ((selection_start < range_start && selection_end > range_start) ||
+		    (selection_end > range_start && selection_end < range_end))
+			sel=TRUE;
+
+		if (!sel) {
+			g_signal_handlers_block_by_func (name_selector_entry, user_insert_text, name_selector_entry);
+			g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+
+			gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry), range_start, range_end);
+
+			ttext = sanitize_string (email);
+			gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), ttext, -1, &range_start);
+			g_free (ttext);
+
+			g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+			g_signal_handlers_unblock_by_func (name_selector_entry, user_insert_text, name_selector_entry);
+
+		}
+		
 		remove_destination_by_index (name_selector_entry, i);
+	}
 	
 	/* Do the actual deletion */
 	
-	
+	if (end_pos == start_pos +1 &&  index_end == index_start) {
+		/* We could be just deleting the empty text */
+		char *c;
+
+		/* Get the actual deleted text */
+		c = gtk_editable_get_chars (GTK_EDITABLE (name_selector_entry), start_pos, start_pos+1);
+
+		if ( c[0] == ' ') {
+			/* If we are at the beginning or removing junk space, let us ignore it */
+			del_space = TRUE;
+		}
+	} else 	if (end_pos == start_pos +1 &&  index_end == index_start+1) {
+		/* We could be just deleting the empty text */
+		char *c;
+
+		/* Get the actual deleted text */
+		c = gtk_editable_get_chars (GTK_EDITABLE (name_selector_entry), start_pos, start_pos+1);
+
+		if ( c[0] == ',' && !is_quoted_at (text, start_pos)) {
+			/* If we are at the beginning or removing junk space, let us ignore it */
+			del_comma = TRUE;
+		}
+	}
+
+	if (del_comma) {
+		int range_start=-1, range_end;
+		EDestination *dest = find_destination_by_index (name_selector_entry, index_end);
+		/* If we have deleted the last comma, let us autocomplete normally
+		 */
+		
+		if (dest && len - end_pos  != 0) {
+			
+			EDestination *destination1  = find_destination_by_index (name_selector_entry, index_start);
+			char *ttext;
+			const char *email=NULL;
+
+			if (destination1)
+				email = e_destination_get_address (destination1);
+
+			if (email && *email) {
+
+				if (!get_range_by_index (text, i, &range_start, &range_end)) {
+					g_warning ("ENameSelectorEntry is out of sync with model!");
+					return;
+				}
+
+				g_signal_handlers_block_by_func (name_selector_entry, user_insert_text, name_selector_entry);
+				g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+
+				gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry), range_start, range_end);
+
+				ttext = sanitize_string (email);
+				gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), ttext, -1, &range_start);
+				g_free (ttext);
+
+				g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+				g_signal_handlers_unblock_by_func (name_selector_entry, user_insert_text, name_selector_entry);
+			}
+			
+			if (range_start != -1) {
+				start_pos = range_start;
+				end_pos = start_pos+1;
+				gtk_editable_set_position (GTK_EDITABLE (name_selector_entry),start_pos);
+			}
+		}
+	}
 	gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry),
 				  start_pos, end_pos);
 	
@@ -1243,7 +1320,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 		/* If the entry was completely cleared, remove the initial destination too */
 		remove_destination_by_index (name_selector_entry, 0);
 		generate_attribute_list (name_selector_entry);
-	} else {
+	} else  if (!del_space){
 		modify_destination_at_position (name_selector_entry, start_pos);
 	}
 
@@ -1257,7 +1334,6 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 		name_selector_entry->type_ahead_complete_cb_id = 0;
 	}
 	
-end_of_user_delete_text:
 	g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
 }
 
@@ -1294,6 +1370,7 @@ completion_match_selected (ENameSelectorEntry *name_selector_entry, GtkTreeModel
 	/* Place cursor at end of address */
 
 	gtk_editable_set_position (GTK_EDITABLE (name_selector_entry), cursor_pos);
+	g_signal_emit (name_selector_entry, signals[UPDATED], 0, destination, NULL);
 	return TRUE;
 }
 
@@ -1302,21 +1379,66 @@ entry_activate (ENameSelectorEntry *name_selector_entry)
 {
 	gint         cursor_pos;
 	gint         range_start, range_end;
-	const gchar *text;
 	ENameSelectorEntryPrivate *priv;
+	EDestination  *destination;
+	gint           range_len;
+	const gchar   *text;
+	gchar         *cue_str;
+	
+	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (name_selector_entry));
+	if (cursor_pos < 0)
+		return;
 
 	priv = E_NAME_SELECTOR_ENTRY_GET_PRIVATE (name_selector_entry);
 
-	/* Show us what's really there */
+	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
+	get_range_at_position (text, cursor_pos, &range_start, &range_end);
+	range_len = range_end - range_start;
+	if (range_len < COMPLETION_CUE_MIN_LEN)
+		return;
 
-	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (name_selector_entry));
+	destination = find_destination_at_position (name_selector_entry, cursor_pos);
+
+	cue_str = get_entry_substring (name_selector_entry, range_start, range_end);
+#if 0	
+	if (!find_existing_completion (name_selector_entry, cue_str, &contact,
+				       &textrep, &matched_field)) {
+		g_free (cue_str);
+		return;
+	}
+#endif	
+	g_free (cue_str);	
 	sync_destination_at_position (name_selector_entry, cursor_pos, &cursor_pos);
 
 	/* Place cursor at end of address */
-
-	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	get_range_at_position (text, cursor_pos, &range_start, &range_end);
+
+	if (priv->is_completing) {
+		char *str_context=NULL;
+		
+		str_context = gtk_editable_get_chars (GTK_EDITABLE (name_selector_entry), range_end, range_end+1);
+		
+		if (str_context[0] != ',') {
+			/* At the end*/
+			gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), ", ", -1, &range_end);
+		} else {
+			/* In the middle */
+			int newpos = strlen (text);
+
+                        /* Doing this we can make sure that It wont ask for completion again. */
+			gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), ", ", -1, &newpos);
+			g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+			gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry), newpos-2, newpos);
+			g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+
+			/* Move it close to next destination*/
+			range_end = range_end+2;
+			
+		}
+	}
+
 	gtk_editable_set_position (GTK_EDITABLE (name_selector_entry), range_end);
+	g_signal_emit (name_selector_entry, signals[UPDATED], 0, destination, NULL);
 	
 	priv->is_completing = FALSE;
 }
@@ -1341,6 +1463,14 @@ user_focus_out (ENameSelectorEntry *name_selector_entry, GdkEventFocus *event_fo
 	if (!event_focus->in && priv->is_completing) {
 		entry_activate (name_selector_entry);
 	}
+	
+	if (name_selector_entry->type_ahead_complete_cb_id) {
+		g_source_remove (name_selector_entry->type_ahead_complete_cb_id);
+	}
+
+	if (name_selector_entry->update_completions_cb_id) {
+		g_source_remove (name_selector_entry->update_completions_cb_id);
+}
 
 	clear_completion_model (name_selector_entry);
 
@@ -1519,8 +1649,10 @@ destination_row_changed (ENameSelectorEntry *name_selector_entry, GtkTreePath *p
 	n = gtk_tree_path_get_indices (path) [0];
 	destination = e_destination_store_get_destination (name_selector_entry->destination_store, iter);
 
+	if (!destination)
+		return;
+	
 	g_assert (n >= 0);
-	g_assert (destination != NULL);
 
 	entry_text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	if (!get_range_by_index (entry_text, n, &range_start, &range_end)) {
@@ -1795,6 +1927,7 @@ editor_closed_cb (GtkObject *editor, gpointer data)
 	g_free (contact_uid);
 	g_object_unref (contact);
 	g_object_unref (editor);
+	g_object_unref (name_selector_entry);
 }
 
 static void
@@ -1866,6 +1999,7 @@ popup_activate_contact (ENameSelectorEntry *name_selector_entry, GtkWidget *menu
 			return;
 
 		contact_list_editor = (*name_selector_entry->contact_list_editor_func) (book, contact, FALSE, TRUE);
+		g_object_ref (name_selector_entry);
 		g_signal_connect (contact_list_editor, "editor_closed",
 				  G_CALLBACK (editor_closed_cb), name_selector_entry);
 	} else {
@@ -1875,6 +2009,7 @@ popup_activate_contact (ENameSelectorEntry *name_selector_entry, GtkWidget *menu
 			return;
 
 		contact_editor = (*name_selector_entry->contact_editor_func) (book, contact, FALSE, TRUE);
+		g_object_ref (name_selector_entry);
 		g_signal_connect (contact_editor, "editor_closed",
 				  G_CALLBACK (editor_closed_cb), name_selector_entry);
 	}
@@ -2085,6 +2220,16 @@ e_name_selector_entry_init (ENameSelectorEntry *name_selector_entry)
 	  return;
   }
 
+  /* read minimum_query_length from gconf*/
+  GConfClient *gconf;
+  gconf = gconf_client_get_default();
+  if (COMPLETION_CUE_MIN_LEN == 0) {
+	  if ((COMPLETION_CUE_MIN_LEN = gconf_client_get_int (gconf, MINIMUM_QUERY_LENGTH, NULL)))
+		;
+	  else COMPLETION_CUE_MIN_LEN = 3;
+  }
+  g_object_unref (G_OBJECT (gconf));
+
   /* Edit signals */
 
   g_signal_connect (name_selector_entry, "insert-text", G_CALLBACK (user_insert_text), name_selector_entry);
